@@ -30,6 +30,7 @@
 #
 #------------------------------------------------------------------------
 # TODO:
+# - * Add support for paragraph-mode sorting (e.g., CR-encoding).
 # - Rework using main.py
 # - Add sanity check for disk space issues.
 # - Have streamlined version just using output from sort.
@@ -60,7 +61,7 @@ RANDOM_SEED = tpo.getenv_integer("RANDOM_SEED", 15485863,
 
 
 class Dummy_Main(Main):
-    """Class for reading input using """
+    """Class for reading input using Main"""
     
     def __init__(self, input_stream):
         super().__init__(runtime_args=[])
