@@ -515,10 +515,18 @@ class stdin_reader(object):
         return line
 
 
+def read_all_stdin():
+    """Read all STDIN and return as a string"""
+    data = ''.join(sys.stdin.readlines()) if not sys.stdin.isatty() else ''
+    debug.trace(debug.VERBOSE, f'read_all_stdin() => "{data}"')
+    return data
+
+
 def read_entire_file(filename, **kwargs):
     """Read all of FILENAME and return as a string
     Note: optional arguments to open() passed along (e.g., encoding amd error handling)"""
-    ## EX: (write_file("/tmp/fu", "1\n2\n3\n"), read_entire_file("/tmp/fu"))[1] => "1\n2\n3\n"
+    # TODO: allow for overriding handling of newlines (e.g., block \r being treated as line delim)
+    # EX: write_file("/tmp/fu123", "1\n2\n3\n"); read_entire_file("/tmp/fu123") => "1\n2\n3\n"
     data = ""
     try:
         ## OLD: with open(filename) as f:
@@ -536,6 +544,20 @@ def read_entire_file(filename, **kwargs):
 read_file = read_entire_file
 
 
+def read_lines(filename):
+    """Return lines in FILENAME as list (each without newline)"""
+    # EX: read_lines("/tmp/fu123.list") => ["1", "2", "3"]
+    # note: The final newline is ignored, s
+    contents = read_entire_file(filename)
+    lines = contents.split("\n")
+    if ((lines[-1] == "") and contents.endswith("\n")):
+        lines = lines[:-1]
+    debug.trace(7, f"read_lines({filename}) => {lines}")
+    return lines
+#
+# EX: l = ["1", "2"]; f="/tmp/12.list"; write_lines(f, l); read_lines(f) => l
+
+
 def read_binary_file(filename):
     """Read FILENAME as byte stream"""
     debug.trace_fmt(7, "read_binary_file({f}, _)", f=filename)
@@ -558,6 +580,7 @@ def read_directory(directory):
     debug.trace_fmtd(5, "read_directory({d}) => {r}", d=directory, r=files)
     return files
 
+
 def get_directory_filenames(directory, just_regular_files=False):
     """Returns full pathname for files in DIRECTORY, optionally restrictded to JUST_REGULAR_FILES
     Note: The files are returned in lexicographical order"""
@@ -572,6 +595,7 @@ def get_directory_filenames(directory, just_regular_files=False):
     debug.trace_fmtd(5, "get_directory_filenames({d}) => {r}", d=directory, r=files)
     return files
     
+
 def read_lookup_table(filename, skip_header=False, delim=None, retain_case=False):
     """Reads FILENAME and returns as hash lookup, optionally SKIP[ing]_HEADER and using DELIM (tab by default).
     Note: Input is made lowercase unless RETAIN_CASE."""
@@ -591,7 +615,8 @@ def read_lookup_table(filename, skip_header=False, delim=None, retain_case=False
                 if (skip_header and (line_num == 1)):
                     continue
                 ## OLD: line = from_utf8(line)
-                line = from_utf8(line.rstrip())
+                ## OLD: line = from_utf8(line.rstrip())
+                line = from_utf8(line.rstrip("\n"))
                 if not retain_case:
                     line = line.lower()
                 if delim in line:
@@ -649,8 +674,8 @@ def lookup_entry(hash_table, entry, retain_case=False):
     return result
 
                 
-def write_file(filename, text, skip_newline=False):
-    """Create FILENAME with TEXT.
+def write_file(filename, text, skip_newline=False, append=False):
+    """Create FILENAME with TEXT and optionally for APPEND.
     Note: A newline is added at the end if missing unless SKIP_NEWLINE"""
     debug.trace_fmt(7, "write_file({f}, {t})", f=filename, t=text)
     # EX: f = "/tmp/_it.list"; write_file(f, "it"); read_file(f) => "it\n"
@@ -658,7 +683,8 @@ def write_file(filename, text, skip_newline=False):
     try:
         if not isinstance(text, STRING_TYPES):
             text = to_string(text)
-        with open(filename, "w") as f:
+        mode = 'a' if append else 'w'
+        with open(filename, mode) as f:
             f.write(to_utf8(text))
             if not text.endswith("\n"):
                 if not skip_newline:
@@ -667,6 +693,8 @@ def write_file(filename, text, skip_newline=False):
         debug.trace_fmtd(1, "Error: Problem writing file '{f}': {exc}",
                          f=filename, exc=get_exception())
     return
+#
+# EX: write_file(f, "new", append=True); read_file(f) => "itnew\n"
 
 
 def write_binary_file(filename, data):
@@ -882,6 +910,7 @@ def to_string(text):
 #
 to_text = to_string
 
+
 def chomp(text, line_separator=os.linesep):
     """Removes trailing occurrence of LINE_SEPARATOR from TEXT"""
     # EX: chomp("abc\n") => "abc"
@@ -895,6 +924,14 @@ def chomp(text, line_separator=os.linesep):
     return result
 
 
+def normalize_dir(path):
+    """Normalize the directory PATH (e.g., removing ending path delim)"""
+    # EX: normalize_dir("/etc/") => "/etc")
+    result = chomp(path, os.path.sep)
+    debug.trace(6, f"normalize_dir({path}) => {result}")
+    return result
+
+
 def non_empty_file(filename):
     """Whether file exists and is non-empty"""
     non_empty = (file_exists(filename) and (os.path.getsize(filename) > 0))
@@ -1073,7 +1110,8 @@ PRECISION = getenv_int("PRECISION", 6,
                        "Precision for rounding (e.g., decimal places)")
 #
 def round_num(value, precision=None):
-    """Round VALUE [to PRECISION places, {p} by default]""".format(p=PRECISION)
+    """Round VALUE [to PRECISION places, 6 by default]"""
+    ## BAD: """Round VALUE [to PRECISION places, {p} by default]""".format(p=PRECISION)
     # EX: round_num(3.15914, 3) => 3.159
     if precision is None:
         precision = PRECISION
@@ -1086,7 +1124,7 @@ def round_num(value, precision=None):
 def round_as_str(value, precision=PRECISION):
     """Returns round_num(VALUE, PRECISION) as string"""
     # EX: round_as_str(3.15914, 3) => "3.159"
-    result = str(round_num(value, precision))
+    result = f"{round_num(value, precision):.{precision}f}"
     debug.trace_fmtd(8, "round_as_str({v}, [prec={p}]) => {r}",
                      v=value, p=precision, r=result)
     return result
@@ -1126,7 +1164,7 @@ def python_maj_min_version():
 def get_args():
     """Return command-line arguments (as a list of strings)"""
     result = sys.argv
-    debug.trace_fmtd(7, "get_args() => {r}", r=result)
+    debug.trace_fmtd(6, "get_args() => {r}", r=result)
     return result
 
 def init():
