@@ -11,6 +11,7 @@
 # - Calculates accuracy as number of agreements over number of cases:
 #   ex:    (tp + tn) / (tp + tn + fp + fn)    for binary classifications
 # - See https://en.wikipedia.org/wiki/Evaluation_of_binary_classifiers#Single_metrics.
+# - Also see https://en.wikipedia.org/wiki/Accuracy_and_precision.
 # - Keep changes in sync with text_categorizer.py (e.g., XGBoost and GPU options).
 # - CherryPy Web server based on following tutorial
 #     https://simpletutorials.com/c/2165/How%20to%20Create%20a%20Simple%20JSON%20Service%20with%20CherryPy
@@ -113,7 +114,7 @@ XGB_VERBOSITY = getenv_int("XGB_VERBOSITY", 0, "Degree of verbosity from 0 to 3"
 USE_LR = system.getenv_bool("USE_LR", False)
 
 # Options for GPU usage
-GPU_DEVICE = system.getenv_value("GPU_DEVICE", "",
+GPU_DEVICE = system.getenv_value("GPU_DEVICE", None,     # TODO: clarify value to use
                                 "Device number for GPU (e.g., shown under nvidia-smi)")
 
 # Options for TFIDF transformation
